@@ -1,4 +1,4 @@
-/* $XFree86$ */ /* -*- mode: c; c-basic-offset: 3 -*- */
+/* -*- mode: c; c-basic-offset: 3 -*- */
 /*
  * Copyright 2000 Gareth Hughes
  * All Rights Reserved.
@@ -29,11 +29,11 @@
  *	Jos Fonseca <j_r_fonseca@yahoo.co.uk>
  */
 
-#include "glheader.h"
-#include "mtypes.h"
-#include "imports.h"
-#include "macros.h"
-#include "colormac.h"
+#include "main/glheader.h"
+#include "main/mtypes.h"
+#include "main/imports.h"
+#include "main/macros.h"
+#include "main/colormac.h"
 
 #include "swrast_setup/swrast_setup.h"
 #include "tnl/t_context.h"
@@ -42,7 +42,6 @@
 #include "mach64_vb.h"
 #include "mach64_ioctl.h"
 #include "mach64_tris.h"
-#include "mach64_state.h"
 
 
 #define MACH64_TEX1_BIT       0x1
@@ -620,7 +619,7 @@ void mach64InitVB( GLcontext *ctx )
    mach64ContextPtr mmesa = MACH64_CONTEXT(ctx);
    GLuint size = TNL_CONTEXT(ctx)->vb.Size;
 
-   mmesa->verts = (GLubyte *)ALIGN_MALLOC(size * 4 * 16, 32);
+   mmesa->verts = (GLubyte *)_mesa_align_malloc(size * 4 * 16, 32);
 
    {
       static int firsttime = 1;
@@ -636,7 +635,7 @@ void mach64FreeVB( GLcontext *ctx )
 {
    mach64ContextPtr mmesa = MACH64_CONTEXT(ctx);
    if (mmesa->verts) {
-      ALIGN_FREE(mmesa->verts);
+      _mesa_align_free(mmesa->verts);
       mmesa->verts = 0;
    }
 }
